class Page < ActiveRecord::Base
  acts_as_list

  attr_translatable! :title
  attr_translatable :body, :description

  default_scope { order(:position) }

  scope :active, -> { where(enabled: true) }
  scope :pages, -> { where(mode: :page) }
  scope :anchors, -> { where(mode: :anchor) }

  def path
    link
  end

  def path=(value)
    self.link = value
  end

  def url
    url_id || id
  end

  def self.modes
    %w(path link page anchor)
  end

  def self.links
    {
        main: :root_path,
        contact_us: :contact_us_path
    }
  end

  def self.find_by_url(url_id)
    Page.find_by(url_id: url_id) || Page.find_by(id: url_id)
  end
end
