class Phrase < ActiveRecord::Base
  has_many :translations, dependent: :destroy
  has_many :phrase_containers

  accepts_nested_attributes_for :translations

  after_create :create_translations
  after_save :update_translations

  def text(*args)
    set_params_by_args *args
    value = nil
    if texts_cache.has_key? @locale
      value = texts_cache[@locale]
    else
      translation = get_translation
      value = translation.text if translation
    end

    if value.blank? and @seek_closest_locale
      translation = get_closest_translation
      if translation and translation.text.present?
        if translation.locale == @locale
          value = texts_cache[@locale] = translation.text
        else
          value = @show_symbol ? "#{translation.locale.upcase}: #{translation.text}" : translation.text
        end
      end
    end

    @locale = nil
    value
  end

  def text=(text)
    set_params_by_args
    translation = get_or_build_translation
    translation.text = text
    changed_translations[@locale] = translation
    texts_cache[@locale] = text
    @locale = nil
  end

  def texts_cache
    @texts ||= {}
  end

  def reload
    texts_cache.clear
    super
  end

  def update_translations
    changed_translations.values.each &:save
    changed_translations.clear
  end

  def locale=(locale)
    @locale = locale.to_s
  end

  def get_closest_translation
    if changed_translations.empty?
      translations.not_blank.localed(@locale).first
      # translations.localed(@locale).first
    else
      all_translations = changed_translations.clone
      translations.each do |t|
        if all_translations[t.locale].blank? and t.text.present?
          all_translations[t.locale] = t
        end
        # all_translations[t.locale] = t unless all_translations.has_key?(t.locale)
      end
      closest_locales_for(@locale).each do |l|
        return all_translations[l] if all_translations[l] and all_translations[l].text.present?
      end
      nil
    end
  end

  # def empty?
  #   any_translation = get_any_translation
  #   any_translation.nil? or not any_translation.text.present?
  # end

  def completed?
    (locales - translations.map(&:locale)).empty?
  end

  def get_translation(locale = nil)
    locale ||= @locale
    changed_translations[locale] || translations.find_by(locale: locale)
  end

  private

  def set_params_by_args(*args)
    if args && args[0] && (args[0].is_a? Hash)
      @locale = args[0][:locale].to_s if args[0].has_key? :locale
      @show_symbol = args[0][:symbol] if args[0].has_key? :symbol
    end
    @seek_closest_locale = @locale.nil?
    @locale ||= I18n.locale.to_s
  end

  def closest_locales_for(locale)
    ([locale] + (locales - [locale]))
  end

  def get_or_build_translation
    get_translation || translations.build(locale: @locale)
  end

  def get_any_translation
    changed_translations.values.first || (translations.where.not(text: ['', nil]).first unless new_record?)
  end

  def create_translations
    changed_translations.values.each { |t| t.update phrase_id: id }
    changed_translations.clear
  end

  def changed_translations
    @changed_translations ||= {}
  end
end
