class PhraseContainer < ActiveRecord::Base
  belongs_to :phrase
  belongs_to :translatable, polymorphic: true

  validates :translatable, presence: true

  scope :for, ->(translatable, attribute_name) { where(translatable: translatable, attribute_name: attribute_name) }

  before_save do
    phrase.save
  end

  before_destroy do
    if phrase.present? && phrase.phrase_containers.count == 1
      phrase.destroy
    end
  end

  def empty?
    phrase.nil? || phrase.empty?
  end

  def text(*args)
    value = phrase.text(*args)
    @locale = nil
    value
  end

  def text=(text)
    @locale ||= I18n.locale.to_s
    if phrase.nil?
      # build_phrase(locale: @locale, text: text) unless set_phrase_by_translation(text)
      build_phrase(locale: @locale, text: text)
    else
      change_phrase(text)
    end
    @locale = nil
  end

  def locale=(locale)
    @locale = locale.to_s
  end

  private

  # def set_phrase_by_translation(text)
  #   translation = Translation.by_locale(@locale).where.not(phrase_id: nil).find_by(text: text)
  #   if translation
  #     self.phrase = translation.phrase
  #     true
  #   else
  #     false
  #   end
  # end

  def change_phrase(text)
    # if phrase.phrase_containers.count < 2 || phrase.get_translation(@locale).nil?
      phrase.locale, phrase.text = @locale, text
    # else
    #   suitable_phrase = find_best_suitable_phrase(text)
    #   if suitable_phrase.present?
    #     take_new_phrase(suitable_phrase)
    #   else
    #     build_new_phrase(text)
    #   end
    # end
  end

  # def find_best_suitable_phrase(text)
  #   max_translations_found = 0
  #   best_suitable_phrase = nil
  #   Translation.by_locale(@locale).where.not(phrase_id: nil, text:['', nil]).where(text: text).each do |some_translation|
  #     some_phrase = some_translation.phrase
  #     current_translations_count = 0
  #     (locales - [@locale]).each do |locale|
  #       t = phrase.get_translation(locale)
  #       if t and t.text.present?
  #         current_translations_count += some_phrase.translations.find_by(text: t.text, locale: t.locale).present? ? 1:-1
  #       end
  #     end
  #     if current_translations_count > max_translations_found
  #       best_suitable_phrase = some_phrase
  #       max_translations_found = current_translations_count
  #     end
  #   end
  #   best_suitable_phrase
  # end

  # def take_new_phrase(new_phrase)
  #   locales.each do |locale|
  #     t = phrase.get_translation(locale)
  #     if t and new_phrase.translations.where.not(text: ['', nil]).find_by(locale: t.locale).nil?
  #       new_phrase.locale = locale
  #       new_phrase.text = t.text
  #     end
  #   end
  #   self.phrase = new_phrase
  # end

  # def build_new_phrase(text)
  #   old_phrase = phrase
  #   build_phrase
  #   locales.each do |locale|
  #     phrase.locale = locale
  #     if @locale == locale
  #       phrase.text = text
  #     else
  #       t = old_phrase.get_translation(locale)
  #       phrase.text = t.text if t and t.text.present?
  #     end
  #   end
  #   phrase
  # end

end
