# noinspection RubyClassVariableUsageInspection
class Setting < ActiveRecord::Base
  validates :name, presence: true, uniqueness: true

  before_update :reset_setting_variable

  default_scope {order :name}

  DEFAULTS = {
      title: {default: 'Blank site', format: :string},
      description: {default: 'New blank site - it is stub for future site!', format: :string},
      keywords: {default: '', format: :string},
      favicon: {default: nil, format: :image},
      gse_id: {default: '007392588842326689376:3onw-lza4mc', format: :string},
      google_analytics_id: {default: 'UA-63039804-12', format: :string},
      stk_node_address: {default: 'znkHBbvPzgbB3zNmqDFZK3eWCSM7j7VfUpm', format: :string}
  }

  def self.init
    DEFAULTS.keys.each {|name| get(name)[:default]}
    self
  end

  def self.get(name)
    setting = find_or_initialize_by(name: name)
    setting.update(value: DEFAULTS[name.to_sym][:default]) if setting.new_record?
    setting
  end

  def self.title
    @@title ||= get(:title).real_value
  end

  def self.description
    @@description ||= get(:description).real_value
  end

  def self.keywords
    @@keywords ||= get(:keywords).real_value
  end

  def self.favicon
    @@favicon ||= get(:favicon).real_value
  end

  def self.gse_id
    @@gse_id ||= get(:gse_id).real_value
  end

  def self.google_analytics_id
    @@google_analytics_id ||= get(:google_analytics_id).real_value
  end

  def self.stk_node_address
    @@stk_node_address ||= get(:stk_node_address).real_value
  end


  def real_value
    case format
    when :image
      image = Image.find_by(id: value)
      unless image
        image = Image.create
        update value: image.id
      end
      image
    when :int
      value.to_i
    when :bool
      value.to_i > 0
    when :array
      value.split
    when :hash
      Hash[*value.split(/,\s*/).map {|s| p = s.split(/:\s*/); [p[0].to_sym, p[1]]}.flatten]
    else
      value
    end
  end

  def formatted_value
    case format
    when :int
      value.to_i
    when :bool
      I18n.t("humanizers.bool.#{real_value}")
    when :image
      "<img src='#{real_value.data_url}'/>".html_safe
    else
      value
    end
  end

  def format
    setting_default = DEFAULTS[name.to_sym]
    setting_default ? setting_default[:format] : nil
  end

  private

  def reset_setting_variable
    self.class.class_variable_set "@@#{name}", nil
  end
end
